/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



/**
 * Application Widget, this will take the events from SDL and
 * send them around
 */
 
#ifndef APPLICATION__H_
#define APPLICATION__H_
 
#include "widget.h"
#include "screensurface.h"

class Application : public Widget {
  public:
  
    /**
     * initialized the SDL library but does NOT set a graphic mode
     */
    Application();

    /**
     * closes the SDL library
     */
    ~Application();
    
    /**
     * sets a video mode
     */
    void setVideoMode(Uint16 xRes, Uint16 yRes, Uint8 bpp);
    
    /**
     * switches between Windowd and Fullscreen mode
     */
    void toggleFullscreen();
    
    /**
     * returns true if currently in Fullscreen mode
     */
    bool isFullscreen();
    
    /**
     * this function must be called after the constructor.
     * here you can put all the things you need: setup widgets, load data, ...
     * you must also setup a video mode within this function (otherwise you
     * will receive an error message when starting
     */
    virtual void init() { }
    
    /**
     * this function is called while destructing the application here you can
     * destroy all the data you have creates within the init procedure
     */
    virtual void done() { }
    
    /**
     * starts the event queue.
     * this function will not return until the application finishes
     */
    void run();
    
    /**
     * call this function inside your event handler to make the
     * event queue finish looping
     */
    void quit();
    
  protected:
  
    /**
     * the surface that represents the screen
     */
    ScreenSurface *screen;

    /**
     * if this variable becomes true, the eventloop will be left 
     * as soon as possible
     */    
    bool quitApplication;
};

#endif