/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "background.h"

Background::Background(istream &in) {
  Uint16 w, h;
  in.read(&w, 2);
  in.read(&h, 2);
  
  if ((w != 0) && (h != 0)) {
    image = new Surface(w, h, Color::getTrueColor()?Pixelformat(32):Pixelformat(8));
    image->loadImage(in);
  } else {
    image = NULL;
    color = Color(in);
  }
}

Background::~Background() {
  if (image != NULL) {
    delete image;
    image = NULL;
  }
}

void Background::draw(Surface &dest, const Rect &area) {
  if (image == NULL) {
    dest.setClipRect(area);
    dest.bar(area, color);
  } else {
    // tiled output of background image
    
    Uint16 x, y;
    x = y = 0;
    while (x + image->width() < area.getUpperLeft().getX()) x += image->width();
    while (y + image->height() < area.getUpperLeft().getY()) y += image->height();
    
    Uint16 xs = x;
    
    dest.setClipRect(area);
    
    do {
      x = xs;
    
      do {
        image->blit(dest, Point(x, y));
      
        x += image->width();
      } while (x <= area.getLowerRight().getX());
      
      y += image->height();
    } while (y <= area.getLowerRight().getY());
  }
}