/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "board.h"

#include <assert.h>

Board::Board(Uint8 sz) {
  assert(sz <= MAXBOARDSIZE);
  size = sz;
  board = new unsigned char[sz*sz];
  
  clearBoard();
}

void Board::clearBoard() {
  for (int x = 0; x < size; x++)
    for (int y = 0; y < size; y++)
      board[y*size+x] = EMPTY;
}

Board::~Board() {
  delete [] board;
}

Uint8 Board::getVertex(const Vertex &v) const {
  assert(v.col() < size);
  assert(v.row() < size);
  return board[v.row()*size + v.col()];
}

void Board::setVertex(const Vertex &v, const Uint8 val) {
  assert(v.col() < size);
  assert(v.row() < size);
  assert((val == EMPTY) || (val == BLACK) || (val == WHITE));
  board[v.row()*size + v.col()] = val;
}