/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



/**
 * encapsulates go boards of different sizes.
 * this is not more that a square array where each position
 * can have 3 states: empty, black or white (for now)
 * it includes asserts for the functions to assure that you don't
 * overrun the borders
 */
 
#ifndef BOARD__H_
#define BOARD__H_

#include "vertex.h"

#include <SDL/SDL.h>

#include <string>

static const Uint8 MAXBOARDSIZE = 21;

class Board {

  public:
  
    /// creates a board of size sz x sz, all the fiels will be empty
    Board(const Uint8 sz);
  
    /// releases the memory reserved
    ~Board();
  
    /// value used to represent empty fields
    static const Uint8 EMPTY = 0;

    /// value used to represent fields with a black stone
    static const Uint8 BLACK = 1;

    /// value used to represent fields with a white stone
    static const Uint8 WHITE = 2;
  
    /// sets all the fields to Board::EMPTY
    void clearBoard();
  
    /// returns the value of the field at the given vertex
    Uint8 getVertex(const Vertex &v) const;
  
    /// sets the field at the given vertex
    void setVertex(const Vertex &v, Uint8 val);
    
    /// returns the size of the board
    Uint8 getSize() const { return size; }
  
  private:
  
    /// the boarddata
    Uint8 *board;
  
    /// the size of the board
    Uint8 size;
};

#endif