/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



/**
 * encapsulates the board itself.
 * lines (precalculated and self calculated)
 * and the stones
 */
 
#ifndef BOARDLINES__H_
#define BOARDLINES__H_ 
 
#include "vertex.h" 
#include "surface.h"
#include "color.h"

#include <SDL/SDL.h>

#include <iostream.h>

class BoardLines {

  public:

    /**
     * sets the board up with the values insode the stream
     */
    BoardLines(istream &in);
     
    /**
     * frees all reserver memory
     */
    ~BoardLines();
    
    /**
     * selects a boardsize for automatic boards, deletes the current image (if one)
     */	  
    void setBoardSize(Uint8 size);
    
    /**
     * selects a boardsize using the data inside the stream
     * this data must fit the give size
     */
    void setBoardSize(istream &in, Uint8 size);
    
    /**
     * returns the x coodinate of middlepoint of the stone on vertex v (only for automatic
     * boards, if you are using special boards this function will leave the program
     * with an error
     */
    Sint16 xPos(const Vertex &v) const;

    /**
     * returns the y coodinate of middlepoint of the stone on vertex v (only for automatic
     * boards, if you are using special boards this function will leave the program
     * with an error
     */
    Sint16 yPos(const Vertex &v) const;

    /**
     * returns the redius of the stone on vertex v (only for automatic
     * boards, if you are using special boards this function will leave the program
     * with an error
     */
    Sint16 rad(const Vertex &v) const;
    
    /**
     * redraws this part of the board
     * assumes that below is only background
     */
    void draw(Surface &dest, const Rect &r);
    
  private:
  
    /// data for automatic boards
    Sint16 *dat;
    
    /// actual boardsize
    Uint8 size;
    
    /// the boards corners (for automatic boards)
    Sint16 corners[2][4];
    
    /// the colors lines are drawn with
    Color lineColor;
    
    /// the board image (for special boards)
    Surface *lines;
    
    /// the position of the surface lines on screen
    Point imagePos;
    
    /// the thickness of the lines for the actual board
    Uint8 thickness;
};

#endif