/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



/**
 * represent a color with 4 components
 */

#ifndef COLOR__H_
#define COLOR__H_

#include <SDL/SDL.h>

#include <iostream.h>

class Color {
  public:
    /**
     * initializes to transparent color
     */
    Color() : r(0), g(0), b(0), a(0) { }
    Color(Uint8 red, Uint8 green, Uint8 blue, Uint8 alpha) : r(red), g(green), b(blue), a(alpha) { }
    Color(Uint8 col) : i(col) { lookupPalette(); }
    Color(istream &in) ;
  
    Uint8 red() const { return r; }
    Uint8 green() const { return g; }
    Uint8 blue() const { return b; }
    Uint8 alpha() const { return a; }
    Uint8 index() const { return i; }
    
    void setAlpha(Uint8 val) { a = val; }
    
    operator SDL_Color() const;
    
    // always 256 colors
    static void setPalette(SDL_Color *colors);
    static void setTrueColor();
    
    static bool getTrueColor() { return trueColor; }
    static SDL_Color *getPalette() { return palette; }
  
  private:
    static int trueColor;
    static SDL_Color *palette;
    Uint8 r, g, b, a, i;
    
    void lookupPalette();
};

#endif
