/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "duplexpipe.h"

// for pipe, fork, execl, close, dup2 command
#include <unistd.h>
// for waitpid command
#include <sys/wait.h>
#include <fstream>

DuplexPipe::DuplexPipe(const string &command) {

  pid = 0;
  
  // will contain the filedescriptors of the 2 pipes
  int reader[2], writer[2];
  
  // create pipes
  if (pipe(reader)) return;
  if (pipe(writer)) {
    // if failed close first pipe
    close(reader[0]);
    close(reader[1]);
    return;
  }
  
  pid = fork();
  if (pid < 0) {
    // fork failed -> cloes pipe, reset PID
    pid = 0;
    close(reader[0]);
    close(reader[1]);
    close(writer[0]);
    close(writer[1]);
    return;
  }
  
  if (pid == 0) {
    // the child
    
    // set the stdin and stdout to the pipes
    dup2(reader[1], STDOUT_FILENO);
    dup2(writer[0], STDIN_FILENO);
    
    // close the other ends of the pipes
    close(reader[0]);
    close(writer[1]);
    
    // execute command
    execl(command.c_str(), "gnugo", "--mode", "gtp", "--quiet", NULL);
    
    // error occured, we can't do anything so just exit
    exit(1);
  } else {
    // the parent
    
    // create the stream
    inf = new ifstream(reader[0]);
    outf = new ofstream(writer[1]);

    // close the unused end of the pipes
    close(reader[1]);
    close(writer[0]);
  }
}

bool DuplexPipe::connected() {
  return ((pid != 0) && (waitpid(pid, NULL, WNOHANG) == 0));
}

DuplexPipe::~DuplexPipe() {
  // wait for child to exit
  if (waitpid(pid, NULL, 0) == pid) {
    delete outf;
    delete inf;
  } else {
    cout << "Error Closing connection to Go Program" << endl;
  }
}

ostream &DuplexPipe::out() const {
  return *outf;
}

istream &DuplexPipe::in() const {
  return *inf;
}