/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



/**
 * a class that opens a full duplex pipe to a new program
 */

#ifndef DUPLEXPIPE__H_
#define DUPLEXPIPE__H_

#include <string>
#include <iostream>

class DuplexPipe {
  public:
  
    /**
     * runs the program specified in command and connects 2 pipes to the
     * stdin and stdout of this program
     */
    DuplexPipe(const string &command);
  
     /**
      * waits for child to finish working
      */
    ~DuplexPipe();
  
    /**
     * returns true if the child is still running
     */
    bool connected();

    /**
     * returns the stream that allows to send data to the childprocess
     */
    ostream &out() const;
    
    /**
     * returns a stream that allows reading the stdout text from the child
     */
    istream &in() const;
  
  private:
  
    /**
     * the 2 streams that are connected to the pipes
     */
    istream *inf;
    ostream *outf;
    
    /**
     * the PID of the child
     */
    int pid;
};

#endif