/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



/**
 * manage board using a program.
 * you need to have at least on program given because only this
 * program knows the exact rules (removes stones...)
 * but otherwise you are free to do whatever you wish
 */
 
#ifndef GAME__H_
#define GAME__H_

#include "vertex.h"
#include "gtp.h"
#include "board.h"
#include "options.h"

#include <SDL/SDL.h>

#include <string>
 
class Game {

  public:
  
    /**
     * starts new game.
     * the strings give the position of the program, if empty a human player
     * is assumed
     */
    Game(const Options &opt);
    ~Game();
  
    /**
     * returns the coordinates of the last move
     */
    Vertex *lastmove() const;

    /**
     * black move my a man 
     */
    bool blackM(const Vertex &v);
    
    /**
     * black move by the computer (only possible if set in constructor)
     */
    void blackC();
    
    /**
     * white move my man
     */
    bool whiteM(const Vertex &v);
    
    /**
     * white move by computer
     */
    void whiteC();
    
    /**
     * the computer functions do not wait for finish of the functions
     * so call this functions to fins out if the last call to
     * blackC or whiteC has been finished you shouldn't call this function
     * if you haven't calles one of the other.
     */
    bool computerFinished();
    
    /**
     * pass for black and white
     */
    void blackP();
    void whiteP();
    
    /**
     * start a new game with the same parameters
     */
    void newGame();
  
    Uint8 getVertex(const Vertex &v) const { return board->getVertex(v); }
    
    const Board *getBoard() const { return board; }
    
    const Vertex getLast() const { return last; }
    
    void loadSGF(string filename);
  
  private:
  
    Vertex last;
    GTP *programB, *programW, *controller, *programs[3];
    Uint8 size;
    Board *board;
    int lastCommand;
    int lastBlack;
    GTP *lastProgram;
  
    /**
     * updates the board using a method of asking as few questions as
     * possible starting from the given vertex: if there are any
     * more changes caused by a change of this vertex they are somewhere
     * around this point
     */
    void updateBoard(const Vertex &v);
};

#endif