/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "gtp.h" 
#include "board.h"

#include <iostream>
#include <strstream>
#include <assert.h>
#include <unistd.h>
#include <fcntl.h>

/**
 * the poller can not be within the object so it needs to get all
 * the necessary information with its pointer
 * this is REALLY dirty code, but the only way it could get working
 * first i switch the blocking for file handlers off the lieds
 * to an i == -1 if there is nothing to read and leaves the procedure
 * then blocking is switched back on so that i really read the whole line
 * and not just a part of it
 */
void GTP::getOutput() {
  char start;
  Uint16 cNumber;
  int i;
  
  filebuf *f = (filebuf*)go->in().rdbuf();
  int fd = f->fd();
  
  while (true) {
    fcntl(fd, F_SETFL, fcntl(fd, F_GETFL, 0) | O_NONBLOCK);
    i = read(fd, &start, 1);
    fcntl(fd, F_SETFL, fcntl(fd, F_GETFL, 0) & ~O_NONBLOCK);
    
    if (i == -1) break;
    
    // if line doesn't start with '=' read till end
    // end restart loop
    if (start != '=') {
      while (start != '\n') read(fd, &start, 1);
      continue;
    }

    // get the command number
    cNumber = 0;
    while (true) {
      read(fd, &start, 1);
      if ((start < '0') || (start > '9')) break;
      cNumber = cNumber * 10 + start - '0';
    }
    
    assert(commands.count(cNumber) == 1);
    
    if (commands[cNumber].cmdForget) {
      commands.erase(cNumber);
      while (start != '\n') read(fd, &start, 1);
    } else {
      while (start != '\n') {
        read(fd, &start, 1);
	if (start != '\n')
          commands[cNumber].cmdResult += start;
      }
    }
  }
}

GTP::GTP(const string &command) {
  go = new DuplexPipe(command);
  
  commandNumber = 0;
}

GTP::~GTP() { }

int GTP::cmd_genmoveWhite() {

  GTPCommand c;
  c.cmdForget = false;
  c.cmdResult = "";
  
  commands[commandNumber] = c;
  
  go->out() << commandNumber << " genmove_white" << endl;
  cout << "o" << commandNumber << " genmove_white" << endl;
  
  return commandNumber++;
}
  
bool GTP::res_genmove(int command, Vertex &v) {

  getOutput();
  string s = commands[command].cmdResult;
    
  if (s != "") {
    cout << "i" << s << endl;
    if (s.compare("PASS") == 0) {
      v = Vertex();
      return true;
    }
    istrstream s(s.c_str());
    s >> v;
    commands.erase(command);
    return true;
  }
  
  return false;
}

int GTP::cmd_genmoveBlack() {

  GTPCommand c;
  c.cmdForget = false;
  c.cmdResult = "";

  commands[commandNumber] = c;
  
  go->out() << commandNumber << " genmove_black" << endl;
  cout << "o" << commandNumber << " genmove_black" << endl;
  
  return commandNumber++;
}

void GTP::moveBlack(const Vertex &v) {
  GTPCommand c;
  c.cmdForget = true;
  c.cmdResult = "";

  commands[commandNumber] = c;

  go->out() << commandNumber << " black " << v << endl;
  cout << "o" << commandNumber << " black " << v << endl;
  
  commandNumber++;
}

void GTP::moveWhite(const Vertex &v) {
  GTPCommand c;
  c.cmdForget = true;
  c.cmdResult = "";

  commands[commandNumber] = c;

  go->out() << commandNumber << " white " << v << endl;
  cout << "o" << commandNumber << " white " << v << endl;
  
  commandNumber++;
}

bool GTP::connected() { return go->connected(); }

void GTP::quit() { 
  go->out() << "quit" << endl;
  cout << "o" << "quit" << endl;
}

bool GTP::cmd_whiteLegal(const Vertex &v) {
  GTPCommand c;
  c.cmdForget = false;
  c.cmdResult = "";

  commands[commandNumber] = c;
  
  go->out() << commandNumber  << " is_legal white " << v << endl;
  cout << "o" << commandNumber  << " is_legal white " << v << endl;
  
  string s;
  bool result = false;;
  
  while (true) {
    getOutput();
    s = commands[commandNumber].cmdResult;
    
    if (s != "") {
      cout << "i" << s;
      if (s == "1") result = true;
      commands.erase(commandNumber);
      break;
    }
  }
  
  commandNumber++;
  
  return result;
}

bool GTP::cmd_blackLegal(const Vertex &v) {
  GTPCommand c;
  c.cmdForget = false;
  c.cmdResult = "";

  commands[commandNumber] = c;

  go->out() << commandNumber << " is_legal black " << v << endl;
  cout << "o" << commandNumber << " is_legal black " << v << endl;
  
  string s;
  bool result = false;;
  
  while (true) {
    getOutput();
    s = commands[commandNumber].cmdResult;
    
    if (s != "") {
      cout << "i" << s << endl;
      if (s == "1") result = true;
      commands.erase(commandNumber);
      break;
    }
  }
  
  commandNumber++;
  
  return result;
}

void GTP::boardsize(Uint8 sz) {
  GTPCommand c;
  c.cmdForget = true;
  c.cmdResult = "";

  commands[commandNumber] = c;

  go->out() << commandNumber << " boardsize " << int(sz) << endl;
  cout << "o" << commandNumber << " boardsize " << int(sz) << endl;
  
  commandNumber++;
}

Uint8 GTP::cmd_getVertex(const Vertex &v) {
  GTPCommand c;
  c.cmdForget = false;
  c.cmdResult = "";

  commands[commandNumber] = c;

  go->out() << commandNumber  << " color " << v << endl;
  cout << "o" << commandNumber  << " color " << v << endl;
  
  string s;
  Uint8 result = 0;
  
  while (true) {
    getOutput();
    s = commands[commandNumber].cmdResult;
    
    if (s != "") {
      cout << "i" << s << endl;
      if (s.compare("empty") == 0) result = Board::EMPTY;
      if (s.compare("black") == 0) result = Board::BLACK;
      if (s.compare("white") == 0) result = Board::WHITE;
      commands.erase(commandNumber);
      break;
    }
  }
  
  commandNumber++;
  
  return result;
}

void GTP::loadSGF(string filename) {
  GTPCommand c;
  c.cmdForget = false;
  c.cmdResult = "";

  commands[commandNumber] = c;

  go->out() << commandNumber << " loadsgf " << filename << endl;
  cout << "o" << commandNumber << " loadsgf " << filename << endl;
  
  commandNumber++;
}