/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "options.h"

#include <iostream>
#include <fstream>
#include <strstream>

/**
 * default values
 */
Options::Options() {
  boardsize = 19;
  blackPlayer = 0;
  whitePlayer = 0;
  computer1 = "";
  computer2 = "";
  computer3 = "";
  handicap = 0;
  controller = 0;
  ruleset = japanese;
}

Options::~Options() {}

void Options::load() {
  string line(getenv("HOME")), var, val;
  line += "/.SDLGo";
  ifstream in(line.c_str());
  int tmp;
  unsigned int p;
  
  while (getline(in, line)) {
    p = line.find(" ");
    var = line.substr(0, p);
    if (line.length() > p)
      val = line.substr(p + 1);
    else
      val = "";
    if        (var == "boardsize") { istrstream(val.c_str()) >> tmp; boardsize = tmp;
    } else if (var == "black")     { istrstream(val.c_str()) >> tmp; blackPlayer = tmp;
    } else if (var == "white")     { istrstream(val.c_str()) >> tmp; whitePlayer = tmp;
    } else if (var == "controller"){ istrstream(val.c_str()) >> tmp; controller = tmp;
    } else if (var == "computer1") { computer1 = val;
    } else if (var == "computer2") { computer2 = val;
    } else if (var == "computer3") { computer3 = val;
    } else if (var == "graphics")  { graphicsFile = val;
    } else if (var == "handicap")  { istrstream(val.c_str()) >> tmp; handicap = tmp;
    } else if (var == "ruleset") {
      if        (val == "japanese") { ruleset = japanese;
      } else if (val == "chinese")  { ruleset = chinese;
      }
    }
  }
}
void Options::save() {
  string line(getenv("HOME"));
  line += "/.SDLGo";
  ofstream out(line.c_str());
  
  out << "boardsize " << boardsize << endl;
  out << "black " << blackPlayer << endl;
  out << "white " << whitePlayer << endl;
  out << "computer1 " << computer1 << endl;
  out << "computer2 " << computer2 << endl;
  out << "ruleset ";
  switch (ruleset) {
    case japanese: out << "japanese" << endl;
    case chinese: out << "chinese" << endl;
    default: out << "unknown" << endl;
  }
  out << "graphics " << graphicsFile << endl;
  out << "handicap " << handicap << endl;
}
    
Uint8 Options::getBoardsize() const { return boardsize; }
Uint8 Options::getHandicap() const { return handicap; }

int Options::getBlackplayer() const { return blackPlayer; }
int Options::getWhiteplayer() const { return whitePlayer; }
int Options::getController() const { return controller; }

const string& Options::getComputer(int i) const { 
  switch (i) {
    case 1: return computer1;
    case 2: return computer2;
    case 3: return computer3;
    default: return empty;
  }
}

const string& Options::getGraphicsFile() const { return graphicsFile; }

rulesets Options::getRulset() const { return ruleset; }

const string Options::empty = "";