/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



/**
 * this class encapsulates all the options for the game
 * is loads these from a file calles ~/.SDLGo ans writes them into this
 * if this file dosn't exist standards are supposed
 */
 
#ifndef OPTIONS__H_
#define OPTIONS__H_

#include <SDL/SDL.h>

#include <string>
 
enum rulesets { 
  japanese, 
  chinese,
  NNGS
};
 
class Options {
  private:
    Uint8 boardsize;
    
    // empty strings mean human player, otherwise the complete path and name for the gtp-
    // able program
    string computer1;
    string computer2;
    string computer3;
    
    static const string empty;
    
    int blackPlayer;
    int whitePlayer;
    int controller;
    
    // the path of the graphics file to use
    string graphicsFile;
    
    rulesets ruleset;
    
    Uint8 handicap;
  public:
    /**
     * puts a default in every value
     */
    Options();
    ~Options();
    
    /**
     * loads options from file ~/.SDLGo
     */
    void load();
    /**
     * save Options into file ~/.SDLGo -> overwrite file
     */
    void save();
    
    /**
     * gets for all the options
     */
    Uint8 getBoardsize() const;
    Uint8 getHandicap() const;
    int getBlackplayer() const;
    int getWhiteplayer() const;
    int getController() const;
    rulesets getRulset() const;
    const string& getGraphicsFile() const;
    const string& getComputer(int) const;
};

#endif