/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "pixelformat.h"

Pixelformat::Pixelformat(Uint8 bitsPerPixel) {
  switch (bitsPerPixel) {
  case 8:
    rloss = 8; gloss = 8; bloss = 8; aloss = 8;
    rshift = 0; gshift = 0; bshift = 0;
    bytesPerPixel = 1;
    break;
  case 15:
    rloss = 3; gloss = 3; bloss = 3; aloss = 8;
    rshift = 10; gshift = 5; bshift = 0;
    bytesPerPixel = 2;
    break;
  case 16:
    rloss = 3; gloss = 2; bloss = 3; aloss = 8;
    rshift = 11; gshift = 5; bshift = 0;
    bytesPerPixel = 2;
    break;
  case 24:
    rloss = 0; gloss = 0; bloss = 0; aloss = 8;
    rshift = 16; gshift = 8; bshift = 0;
    bytesPerPixel = 3;
    break;
  case 32:
    rloss = 0; gloss = 0; bloss = 0; aloss = 0;
    rshift = 16; gshift = 8; bshift = 0; ashift = 24;
    bytesPerPixel = 4;
    break;
  default:
    // FIXME: throw exception
    exit(2);
  }
  
  rmask = (Uint32(0xFF) >> rloss) << rshift;
  gmask = (Uint32(0xFF) >> gloss) << gshift;
  bmask = (Uint32(0xFF) >> bloss) << bshift;
  amask = (Uint32(0xFF) >> aloss) << ashift;
}

Pixelformat::Pixelformat(Uint8 bpp, Uint32 rmask, Uint32 gmask, Uint32 bmask, Uint32 amask = 0) {
  this->bytesPerPixel = bpp;
  if (bpp == 1) {
    rmask = gmask = bmask = amask = 0;
  }
  this->rmask = rmask;
  this->gmask = gmask;
  this->bmask = bmask;
  this->amask = amask;
  
  if (rmask == 0) {
    rloss = 8;
    rshift = 0;
  } else {
    rshift = 0;
    while (((rmask >> rshift) & 1) == 0) rshift++;
    rloss = 0;
    while (((rmask >> (rshift-rloss)) & 1) == 0) rloss++;
  }
  
  if (gmask == 0) {
    gloss = 8;
    gshift = 0;
  } else {
    gshift = 0;
    while (((gmask >> gshift) & 1) == 0) gshift++;
    gloss = 0;
    while (((gmask >> (gshift-gloss)) & 1) == 0) gloss++;
  }
  
  if (bmask == 0) {
    bloss = 8;
    bshift = 0;
  } else {
    bshift = 0;
    while (((bmask >> bshift) & 1) == 0) bshift++;
    bloss = 0;
    while (((bmask >> (bshift-bloss)) & 1) == 0) bloss++;
  }
  
  if (amask == 0) {
    aloss = 8;
    ashift = 0;
  } else {
    ashift = 0;
    while (((amask >> ashift) & 1) == 0) ashift++;
    aloss = 0;
    while (((amask >> (ashift-aloss)) & 1) == 0) aloss++;
  }
}


Uint32 Pixelformat::mapToPixel(const Color& col) const {
  if (bytesPerPixel == 1) return col.index();
  else
    return (((Uint32(col.red()) >> rloss) << rshift) |
            ((Uint32(col.green()) >> gloss) << gshift) |
            ((Uint32(col.blue()) >> bloss) << bshift) |
  	    ((Uint32(col.alpha()) >> aloss) << ashift));
}

Color Pixelformat::mapToColor(Uint32 pixel) const {
  if (bytesPerPixel == 1) return Color(pixel);
  else
    return Color(((pixel & rmask) >> rshift) << rloss,
                 ((pixel & gmask) >> gshift) << gloss,
	         ((pixel & bmask) >> bshift) << bloss,
	         ((pixel & amask) >> ashift) << aloss);
}



