/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



/**
 * allows the definition of a pixel format (how many bit of r,g,b, ...)
 */

#ifndef PIXELFORMAT__H_
#define PIXELFORMAT__H_

#include "color.h"

#include <SDL/SDL.h>

class Pixelformat {
  public:

    /**
     * format using as many bits as specified for the up to 4 color
     * these are some kind of standards
     */
    Pixelformat(Uint8 bitsPerPixel);

    /**
     * if you really need something special use this constructor, but:
     * - no more than 8 bits per channel
     * - only continuous areas of the field are allowed (e.g. never use 101b as mask)
     */
    Pixelformat(Uint8 bytesPerPixel, Uint32 rmask, Uint32 gmask, Uint32 bmask, Uint32 amask = 0);
    ~Pixelformat() {};
    
    Uint32 mapToPixel(const Color& col) const;
    Color mapToColor(Uint32 pixel) const;

    /**
     * returns bytes per pixel
     */
    Uint8 bpp() const { return bytesPerPixel; }
    
    Uint32 rMask() const { return rmask; }
    Uint32 gMask() const { return gmask; }
    Uint32 bMask() const { return bmask; }
    Uint32 aMask() const { return amask; }
  
  private:

    Uint8 bytesPerPixel;
    Uint32 rmask;
    int rshift;
    int rloss;
    Uint32 gmask;
    int gshift;
    int gloss;
    Uint32 bmask;
    int bshift;
    int bloss;
    Uint32 amask;
    int ashift;
    int aloss;
};

#endif