/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "rect.h"

Rect Rect::invalid = Rect(0,0,-1,-1);
Rect Rect::empty = Rect(0,0,0,0);

Rect::Rect( int x, int y, int w, int h) :
  upperLeft(x,y),
  lowerRight(x+w, y+h) { }

Rect::Rect() :
  upperLeft(0,0),
  lowerRight(0,0) { }

Rect::Rect ( const SDL_Rect &rect) :
  upperLeft(rect.x, rect.y),
  lowerRight(rect.x+rect.w, rect.y+rect.h) { }

Rect::Rect(const Point &uL, const Point &lR) :
  upperLeft(uL),
  lowerRight(lR) { }
  
Rect::Rect(const Rect &r) {
  upperLeft = r.getUpperLeft();
  lowerRight = r.getLowerRight();
}

Rect::~Rect() { }

bool Rect::isEmpty() const { return(upperLeft == lowerRight); }
bool Rect::isValid() const { return(lowerRight >= upperLeft); }
bool Rect::contains(const Point &p) const { return (p >= getUpperLeft()) && (p < getLowerRight()); }
bool Rect::contains(const Rect &r) const { return contains(r.getUpperLeft()) && contains(r.getLowerRight()); }

void Rect::translate(const Point &p) {
  upperLeft += p;
  lowerRight += p;
}

void Rect::warp(const Point &newUpperLeft) {
  Point delta = upperLeft - newUpperLeft;
  
  upperLeft -= delta;
  lowerRight -= delta;
}

void Rect::resize(int w, int h) { lowerRight = upperLeft + Point(w,h); }


Rect Rect::unite(const Rect &r) const {
  if(r.isValid() && isValid()) {
    
    if (isEmpty()) return Rect(r);
    if (r.isEmpty()) return Rect(*this);
    
    Point ul(r.upperLeft.getX() < upperLeft.getX()
               ? r.upperLeft.getX()
               : upperLeft.getX(),
             r.upperLeft.getY() < upperLeft.getY()
               ? r.upperLeft.getY() 
               : upperLeft.getY());
      
    Point lr(r.lowerRight.getX() > lowerRight.getX() 
               ? r.lowerRight.getX() 
               : lowerRight.getX(),
             r.lowerRight.getY() > lowerRight.getY() 
               ? r.lowerRight.getY() 
               : lowerRight.getY());
	       
    return Rect(ul, lr);
  } else 
    return invalid;
}

Rect Rect::intersect(const Rect &r) const {
  if(r.isValid() && isValid()) {
    Point ul(r.upperLeft.getX() > upperLeft.getX() 
               ? r.upperLeft.getX() 
               : upperLeft.getX(),
             r.upperLeft.getY() > upperLeft.getY() 
               ? r.upperLeft.getY() 
               : upperLeft.getY());
		 
    Point lr(r.lowerRight.getX() < lowerRight.getX() 
               ? r.lowerRight.getX() 
               : lowerRight.getX(),
             r.lowerRight.getY() < lowerRight.getY() 
               ? r.lowerRight.getY() 
               : lowerRight.getY());
	       
    if((ul.getX() != lr.getX()) || (ul.getY() != lr.getY())) 
      return Rect(ul, lr);
  }
  
  return invalid;
}

SDL_Rect Rect::toSDL() const {
  SDL_Rect r;
  r.x = upperLeft.getX();
  r.y = upperLeft.getY();
  r.w = width();
  r.h = height();
  
  return r;
}