/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



/**
 *
 */
 
#ifndef RECT__H_
#define RECT__H_

#include "point.h"

#include <SDL/SDL.h>

class Rect {
  public:
  
    Rect();
    Rect(const int x, const int y, const int w, const int h);
    Rect(const SDL_Rect &);
    Rect(const Rect &);
    Rect(const Point& upperLeft, const Point& lowerRight);
    
    virtual ~Rect();

    // some special rectangles
    static Rect invalid;
    static Rect empty;
  
    bool isEmpty() const;
    bool isValid() const;
    bool contains(const Point &) const;
    bool contains(const Rect &)  const;

    //move the rectangle by (dx, dy)
    virtual void translate(const Point &p);
    //move the rectangle, specifying new upperleft corner
    virtual void warp(const Point &);
    ///this keeps the upper left corner and moves the lower right one
    virtual void resize(int w, int h);

    Rect unite(const Rect &r) const;
    Rect intersect(const Rect &r) const;

    Point getUpperLeft() const { return upperLeft; }
    Point getLowerRight() const { return lowerRight; }

    int width() const { return lowerRight.getX() - upperLeft.getX(); }
    int height() const { return lowerRight.getY() - upperLeft.getY(); }
    
    SDL_Rect toSDL() const;

    friend bool operator==(const Rect &p, const Rect &q) { 
      return ((p.upperLeft == q.upperLeft) && 
	      (p.lowerRight == q.lowerRight)); 
    }
  
    //comparism by size of area (width * height)
    friend bool operator<(const Rect &p, const Rect &q) { 
      return (p.width()*p.height()) < (q.width()*q.height()); 
    }
  
  protected:
  
    Point upperLeft, lowerRight;
};

#endif
