/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "sdlapplication.h"
#include "vertex.h"
#include "ttfont.h"

#include <iostream.h>

void SDL_Application::init() {
  cout << "This is SDL_Go" << endl << endl;
  SDL_WM_SetCaption("SDL Go", NULL);
  cout << "  loading options..." << endl;
  options.load();
  cout << "  loading datafile..." << endl;
  data = new Datafile(options.getGraphicsFile());
  cout << "  setting up video..." << endl;
  setVideoMode(data->getXResolution(), data->getYResolution(), data->getBitsPerPixel());
  data->setBoardSize(options.getBoardsize());
  game = new Game(options);
  
  // create the buttons
  for (int b = 0; b < 6; b++)
    buttons[b] = new SDL_PushButton(this, data->getButtonRect(b), b, data);
    
  // create new boardwidget
  board = new SDL_Board(this, Rect(0, 0, data->getXResolution(), data->getYResolution()), data);
  
  data->repaint();
  newGame();
  updateScreen();
}

void SDL_Application::done() {
  delete data;
  delete game;
}

void SDL_Application::loadSGF(const string &filename) {
  game->loadSGF(filename);
  data->updateBoard(game);
}

bool SDL_Application::handleEvent(SDL_Event &e) {
  if (e.type == SDL_QUIT) {
    quit();
    return true;
  } else if (e.type == SDL_USEREVENT) {
    if (e.user.code == 0) { // QUIT
      quit();
      return true;
    } else if (e.user.code == 1) { // PASS
      game->blackP();
      data->updateBoard(game);
      nextBlack = false;
      return true;
    } else if (e.user.code == 2) { // NEW
      newGame();
      return true;
    } else if (e.user.code == 11) { // click onto the board
      if (nextBlack) {
        if (options.getBlackplayer() == 0) {
          if (game->blackM( *((Vertex *)e.user.data1))) {
	    data->updateBoard(game);
	    nextBlack = false;
	    blackTime += SDL_GetTicks() - moveStarttime;
	    moveStarttime = SDL_GetTicks();
	  }
	}
      } else {
        if (options.getWhiteplayer() == 0) {
          if (game->whiteM( *((Vertex *)e.user.data1))) {
  	    data->updateBoard(game);
	    nextBlack = true;
	    whiteTime += SDL_GetTicks() - moveStarttime;
	    moveStarttime = SDL_GetTicks();
	  }
	}
      }
      delete (Vertex *)e.user.data1;
      return true;
    }
  } else if (e.type == SDL_KEYUP) {
    // toggle fullscreenmode when pressen ALT-Enter
    if ((e.key.keysym.sym == SDLK_RETURN) && 
        (e.key.keysym.mod && KMOD_LALT)) {
      toggleFullscreen();
      return true;
    }
  }
  
  // here the computer turns are started and evaluated
  if (!computerThinks) {
    if (nextBlack) {
      if (options.getBlackplayer() != 0) {
        game->blackC();
	computerThinks = true;
        nextBlack = false;
      }
    } else {
      if (options.getWhiteplayer() != 0) {
        game->whiteC();
	computerThinks = true;
        nextBlack = true;
      }
    }
  } else {
    if (game->computerFinished()) {
      computerThinks = false;
      data->updateBoard(game);
      if (nextBlack)
	whiteTime += SDL_GetTicks() - moveStarttime;
      else
        blackTime += SDL_GetTicks() - moveStarttime;
      moveStarttime = SDL_GetTicks();
    }
  }
  
  bool handled = Application::handleEvent(e);
  updateScreen(); 
  
  return handled;
}

void SDL_Application::updateScreen() {
  Rect r = data->getUpdateRect();
  if (!r.isEmpty()) {
    data->getSurface()->clearClipRect();
    data->getSurface()->blit(*screen, r, r.getUpperLeft());

    screen->update();
  }
}

void SDL_Application::newGame() {
  while (!game->computerFinished());
  game->newGame();
  data->updateBoard(game);
  blackTime = 0;
  whiteTime = 0;
  moveStarttime = SDL_GetTicks();
  nextBlack = true;
  computerThinks = false;
  passCount = 0;
}