/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



/**
 * this is the application class for the SDLGo program
 */
 
#ifndef SDL_APPLICATION__H_
#define SDL_APPLICATION__H_

#include "application.h"
#include "options.h"
#include "datafile.h"
#include "sdlpushbutton.h"
#include "sdlboard.h"
#include "game.h"

#include <SDL/SDL.h>

class SDL_Application : public Application {
  
  public:
  
    SDL_Application() : Application() { }
    bool handleEvent(SDL_Event &e);
    
    void loadSGF(const string &filename);
    
    virtual void init();  
    virtual void done();
    
  private:
  
    Options options;
    Datafile *data;
    SDL_PushButton * buttons[6];
    SDL_Board *board;
    Game *game;
    
    Uint32 blackTime, whiteTime, moveStarttime;
    
    // is black the one with the next move ?
    bool nextBlack;
    // is the computer actually thinking about a move?
    bool computerThinks;
    
    // repaints the portion on screen that the datafile thinks has changed
    void updateScreen();
    
    // clears the board, sets times to 0, ...
    void newGame();

    // the pass counter (how many continuous passes)
    Uint8 passCount;    
};

#endif