/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "stonedata.h"

#include <assert.h>

StoneData::StoneData() { stones = NULL; }
StoneData::~StoneData() {
  for (int x = 0; x < 2*size*size; x++) {
    delete stones[x].sprite;
    for (int b = 0; b < 8; b++)
      if (stones[x].modificator[b] != NULL) delete stones[x].modificator[b];
  }

  if (stones != NULL) delete [] stones;
}

void StoneData::load(istream &in, const Uint8 sz) {
  if (stones != NULL) delete [] stones;
  
  size = sz;
  stones = new Stone[size*size*2];
  
  Uint16 w, h;
  Sint16 x, y;
  Uint8 modificatorMap;
  
  char c;
  
  for (int s = 0; s < size*size*2; s++) {
    in.read(&w, 2);
    in.read(&h, 2);
    in.read(&x, 2);
    in.read(&y, 2);
    
    stones[s].position = Point(x, y);
    stones[s].sprite = new Surface(w, h, Color::getTrueColor()?Pixelformat(32):Pixelformat(8));
    stones[s].sprite->loadSprite(in);
    
    in.read(&modificatorMap, 1);
    
    for (int b = 0; b < 8; b++) {
      if ((modificatorMap << b) & 0x80) {
        stones[s].modificator[b] = new Surface(w, h, Color::getTrueColor()?Pixelformat(32):Pixelformat(8));
	stones[s].modificator[b]->loadSprite(in);
        in.read(&c, 1); 
        assert(c == 'M');
      } else {
        stones[s].modificator[b] = NULL;
      }
    }
    in.read(&c, 1); 
    assert(c == 'S');
  }
}

const Rect StoneData::draw(const Vertex &v, Surface &dst, const bool black, const Uint8 above, 
                    const Uint8 right, const Uint8 below, const Uint8 left) {
  int p = ((size-v.row()-1)*size+v.col())*2 + ((black) ? 0 : 1);
  
  Stone &s = stones[p];
  
  s.sprite->blit(dst, s.position);
  if ((above == 1) && (s.modificator[0] != NULL)) s.modificator[0]->blit(dst, s.position);
  if ((right == 1) && (s.modificator[1] != NULL)) s.modificator[1]->blit(dst, s.position);
  if ((below == 1) && (s.modificator[2] != NULL)) s.modificator[2]->blit(dst, s.position);
  if ((left  == 1) && (s.modificator[3] != NULL)) s.modificator[3]->blit(dst, s.position);
  if ((above == 2) && (s.modificator[4] != NULL)) s.modificator[4]->blit(dst, s.position);
  if ((right == 2) && (s.modificator[5] != NULL)) s.modificator[5]->blit(dst, s.position);
  if ((below == 2) && (s.modificator[6] != NULL)) s.modificator[6]->blit(dst, s.position);
  if ((left  == 2) && (s.modificator[7] != NULL)) s.modificator[7]->blit(dst, s.position);
 
  return Rect(s.position.getX(), s.position.getY(), s.sprite->width(), s.sprite->height());
}
  
const Vertex StoneData::getVertex(const Point &p) {
  for (int y = 0; y < size; y++)
    for (int x = 0; x < size; x++) {
      Stone &s = stones[((size-y-1)*size+x)*2];
      if ((p > s.position) &&
          (p < Point(s.position.getX()+s.sprite->width(),
	             s.position.getY()+s.sprite->height()))) {
	if (s.sprite->getpixel(p-s.position).alpha() < 128)
  	  return Vertex(y, x);
      }
    }
	
  return Vertex();
}

const Rect StoneData::setMarker(Surface &dest, const Vertex &v, const Color &c) {
  // set the color into the center of the sprite and hope this is the right place
  Stone &s = stones[((size-v.row()-1)*size+v.col())*2];
  Sint16 x = s.position.getX() + s.sprite->width()/2;
  Sint16 y = s.position.getY() + s.sprite->height()/2;
  Sint16 r = s.sprite->width() > s.sprite->height() ? s.sprite->height() : s.sprite->width();
  r /= 6;
  dest.filledCircle(Point(x, y), r, c);
  return Rect(x-r, y-r, r, r);
}

const Rect StoneData::setMarker(Surface &dest, const Vertex &v, Surface *marker) {
  return Rect::empty;
}

const Rect StoneData::getStoneRect(const Vertex &v, const bool black) {
  int p = ((size-v.row()-1)*size+v.col())*2 + ((black) ? 0 : 1);
  Stone &s = stones[p];
  return Rect(s.position.getX(), s.position.getY(), s.sprite->width(), s.sprite->height());
}