/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#ifndef STONEDATA__H_
#define STONEDATA__H_

#include "surface.h"
#include "point.h"
#include "rect.h"
#include "vertex.h"

#include <SDL/SDL.h>

#include <iostream>

/**
 * structure to manages one stone and its modificators
 */
struct Stone {
  Surface *sprite;
  Point position;
  
  Surface *modificator[8];
};

/**
 * this class manages the special board stone data
 */
class StoneData {

  public:
    StoneData();
    ~StoneData();
    
    /**
     * load data from stream assuming boardsize sz
     */
    void load(istream &in, const Uint8 sz);
    /**
     * draw the stone returning the rectange with the changed area.
     */
    const Rect draw(const Vertex &v, Surface &dst, const bool black, 
                    const Uint8 above, const Uint8 right, const Uint8 below, const Uint8 left);

    const Vertex getVertex(const Point &p);
    const Rect setMarker(Surface &dest, const Vertex &v, const Color &c);
    const Rect setMarker(Surface &dest, const Vertex &v, Surface *marker);
    
    const Rect getStoneRect(const Vertex &v, const bool black);
  private:
    Stone *stones;
    Uint8 size;
    
};

#endif
