/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "stones.h"
#include "point.h"

#include <assert.h>

Stones::Stones(istream &in, BoardLines *bl) {
  Uint16 w, h;
  
  // first the sprites for the 2 stones
  in.read(&w, 2);
  in.read(&h, 2);
  if ((w != 0) && (h != 0)) {
    blackStone = new Surface(w, h, Color::getTrueColor?Pixelformat(32):Pixelformat(8));
    whiteStone = new Surface(w, h, Color::getTrueColor?Pixelformat(32):Pixelformat(8));
    blackStone->loadSprite(in);
    whiteStone->loadSprite(in);
  } else {
    blackStone = NULL;
    whiteStone = NULL;
    blackColor = Color(in);
    whiteColor = Color(in);
  }
  
  data = NULL;
  lines = bl;
  
  char c;
  in >> c; assert(c = 'T'); // FIXME:
  
  in.read(&w, 2); in.read(&h, 2);
  
  if ((w != 0) && (h != 0)) {
    lastMoveMarker = new Surface(w, h, Color::getTrueColor?Pixelformat(32):Pixelformat(8));
    lastMoveMarker->loadSprite(in);
  } else {
    lastMoveMarker = NULL;
    lastMoveMarkerColor = Color(in);
  }
}

Stones::~Stones() {
  if (data != NULL) delete data;
  if (blackStone != NULL) delete blackStone;
  if (whiteStone != NULL) delete whiteStone;
}
  
Rect Stones::putStone(Surface &dest, const Vertex &v, bool black, Uint8 above, Uint8 right, Uint8 below, Uint8 left) {
  dest.clearClipRect();
  if (data != NULL) {
    return data->draw(v, dest, black, above, right, below, left);
  } else if (((blackStone != NULL) && (black)) || ((whiteStone != NULL) && (!black))) {
  // FIXME: scale sprite and put it
  } else {
    if (black)
      dest.filledCircle(Point(lines->xPos(v), lines->yPos(v)), lines->rad(v), blackColor);
    else
      dest.filledCircle(Point(lines->xPos(v), lines->yPos(v)), lines->rad(v), whiteColor);
    return Rect(lines->xPos(v)-lines->rad(v), lines->yPos(v)-lines->rad(v), lines->rad(v)*2+1, lines->rad(v)*2+1);
  }
  
  return Rect::empty;
}
    
const Vertex Stones::getVertex(const Point &p) {
  if (data != NULL) {
    return data->getVertex(p);
  } else {
    // search for a middle point of any vertex that is not more
    // that rad away from the given point
    Vertex v;
    Sint16 dx, dy;
    
    for (int y = 0; y < size; y++)
      for (int x = 0; x < size; x++) {
        v = Vertex(y, x);
	dx = lines->xPos(v) - p.getX();
	dy = lines->yPos(v) - p.getY();
	if (sqrt(dx*dx+dy*dy) < lines->rad(v))
	  return v;
      }
    
    return Vertex(-1, -1);
  }
  return Vertex();
}
    
void Stones::setBoardSize(istream &in, const Uint8 sz) {
  if (data != NULL) delete data;
  
  data = new StoneData();

  data->load(in, sz);

  size = sz;
}

Rect Stones::setMarker(Surface &dest, const Vertex &v) {
  if (data != NULL) {
    if (lastMoveMarker == NULL)
      return data->setMarker(dest, v, lastMoveMarkerColor);
    else ; //FIXME
  } else if (lastMoveMarker != NULL) {
   // FIXME
  } else {
    dest.filledCircle(Point(lines->xPos(v), lines->yPos(v)), lines->rad(v) / 4, lastMoveMarkerColor);
    return Rect(lines->xPos(v)-lines->rad(v), lines->yPos(v)-lines->rad(v), 
                lines->rad(v)*2+1, lines->rad(v)*2+1);
  }
  return Rect::empty;
}

Rect Stones::getStoneRect(const Vertex &v, const bool black) {
  if (data != NULL) {
    return data->getStoneRect(v, black);
  } else if (((blackStone != NULL) && (black)) || ((whiteStone != NULL) && (!black))) {
  // FIXME: scale sprite and put it
  } else {
    return Rect(lines->xPos(v)-lines->rad(v), lines->yPos(v)-lines->rad(v), 
                lines->rad(v)*2+1, lines->rad(v)*2+1);
  }
  return Rect::empty;
}