/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



/**
 * this class handles the output of stones onto screen
 * and also calculates the vertex for a given screen position
 */
 
#ifndef STONES__H_
#define STONES__H_
 
#include "boardlines.h"
#include "surface.h"
#include "stonedata.h"
#include "color.h"

#include <SDL/SDL.h>

#include <iostream.h>
 
class Stones {
 
  public:

    Stones(istream &, BoardLines *bl);
    ~Stones();
    
    /**
     * puts the stone onto an EMPTY vertex.
     * the neighbourhood should be as given by above, the returned rect is the
     * area of the output that has been changed
     */
    Rect putStone(Surface &dest, const Vertex &v, bool black, Uint8 above, Uint8 right, Uint8 below, Uint8 left);
    
    /**
     * returns the vertex the given coordinates belong to
     */
    const Vertex getVertex(const Point &p);
    
    void setBoardSize(Uint8 boardsize) { 
      size = boardsize; 
      if (data != NULL) {
        delete data; 
	data = NULL;
      }
    }
    void setBoardSize(istream &in, Uint8 boardsize);
    
    Rect setMarker(Surface &dest, const Vertex &v);
    
    Rect getStoneRect(const Vertex &v, const bool black);
    
  private:
  
    Color blackColor, whiteColor;
    Surface *blackStone, *whiteStone;
    
    /**
     * the connection to the lines object that contains the
     * information about vertives positions on an automatic board
     */
    BoardLines *lines;
    
    /// the data for special stones
    StoneData *data;
    
    /// actual boardsize
    Uint8 size;
    
    Color lastMoveMarkerColor;
    Surface *lastMoveMarker;
};

#endif