/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "ttfont.h"

#include <strstream>

map<string, TTFReference> TTFont::cache;

TTFont::TTFont(string filename, int ptSize) {
  static bool ttEngineInitialized = false;

  if (!ttEngineInitialized) {
    TTF_Init(); // FIXME: look for errors
    ttEngineInitialized = true;
  }
  char c[4];
  ostrstream o(c, 3);
  
  o << ptSize;

  key = filename + "@" + c;
    
  // look, if the searched font is inside the cache
  if (cache.count(key) == 0) {
    // create entry
    TTFReference r;
    r.refcount = 0;
    r.font = TTF_OpenFont(filename.c_str(), ptSize);
    
    cache[key] = r;
  }
  
  cache[key].refcount++;
  font = cache[key].font;
  
  rendered = NULL;
}

TTFont::~TTFont() {
  cache[key].refcount--;
  
  if (cache[key].refcount == 0) cache.erase(key);
  
  if (rendered != NULL) delete rendered;
}
  
Surface *TTFont::render(string text, const Color &c) {
  if (rendered != NULL) delete rendered;
  
  rendered = new Surface(TTF_RenderText_Blended(font, text.c_str(), c));
  
  return rendered;
}
  
Point TTFont::calcSize(string text) const { return Point(); }
  
int TTFont::getLineSkip() const { return 0; }
  
