/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#ifndef TTFONT__H_
#define TTFONT__H_

#include "surface.h"
#include "color.h"

#include <SDL/SDL_ttf.h>

#include <string>
#include <map>

class Surface;

/**
 * a structure with reference count
 */
 
struct TTFReference {
  TTF_Font *font;
  int refcount;
};

/**
 * A special class to encapsulate a true type font.
 * This class will use the TT-Library in the moment you
 * load the first class the true type library is inistialized
 *
 * Fonts are cached: this means you can open the same font twice without
 * wasting memory.
 */
class TTFont {

  public:

    TTFont(string filename, int ptSize);
    ~TTFont();
  
    /// renders the given text, you don't need to care for the surface
    Surface *render(string text, const Color &);
  
    /// returns a Point representing the size of the rendered text
    Point calcSize(string text) const;
  
    /// returns the recommended distance between two lines of text
    int getLineSkip() const;
  
  private:
    
    /// this is the cache for the fonts every font with every size will be
    /// inside this map exactly once
    static map<string, TTFReference> cache;
    
    /// points to one entry inside the cache
    TTF_Font *font;
    
    string key;
    
    /// contains the last rendered Text
    Surface *rendered;
  
};

#endif