/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



/**
 * encapsulates a position on the board.
 * coordinates starting at (0;0) going up to (254; 254)
 * the output string is go coordinate notation and the input string must be
 * go coordinte notaion to get a predictable result
 */
#ifndef VERTEX__H_
#define VERTEX__H_

#include <SDL/SDL.h>

#include <iostream>
 
class Vertex {
  public:
    /**
     * initializes to 255;255 (invalid coordinate)
     */
    Vertex();
    Vertex(Uint8 row, Uint8 col);
    
    Uint8 row() const { return y; }
    Uint8 col() const { return x; }
    
    Vertex left() const { return Vertex(y, x-1); }
    Vertex right() const { return Vertex(y, x+1); }
    Vertex above() const { return Vertex(y-1, x); }
    Vertex below() const { return Vertex(y+1, x); }
    
    
    /**
     * returns true only if the coordinates are a valid point
     */
    bool isValid();
    
    friend bool operator==(const Vertex &v1, const Vertex &v2) {
      return ((v1.x == v2.x) && (v1.y == v2.y));
    }
    
    friend bool operator!=(const Vertex &v1, const Vertex &v2) {
      return ((v1.x != v2.x) || (v1.y != v2.y));
    }
    
  private:
    Uint8 x, y;
    
  friend ostream& operator<<(ostream& o, const Vertex& v);
  friend istream& operator>>(istream& i, Vertex& v);
};

#endif