/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



#include "widget.h"

Widget::Widget(Widget* parentWidget, const Rect & r) : 
  position(r), 
  parent(parentWidget),
  last(NULL) {
  next = this;
  if (parentWidget != NULL) parentWidget->addWidget(this);
}

Widget::~Widget() {
}

void Widget::addWidget(Widget *w) {
  if (last == NULL)
    last = w;
  else {
    w->next = last->next;
    last->next = w;
    last = w;
  }
}
    
bool Widget::handleEvent(SDL_Event &e) {
  if (last == NULL) return false;
  
  Widget *w = last->next;

  do {
    if (w->handleEvent(e)) return true;
    w = w->next;
  } while (w != last->next);

  return false;
}
    
void Widget::processEvent(SDL_Event &e) {
  if (parent != NULL)
    parent->processEvent(e);
  else
    handleEvent(e);
}

bool Widget::onWidget(const Point &p) const {
  Point pt(p);
  Widget *w = parent;
  while (w->parent != NULL) {
    pt -= w->position.getUpperLeft();
    w = w->parent;
  }
  return position.contains(pt);
}