/*
    SDL_Go - a GUI for GnuGo    
    Copyright (C) 2000  Andreas Rver

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    Homepage: http://www.rz.tu-ilmenau.de/~roever/SDL_Go
    Email: andreas_roever@yahoo.com
*/



/**
 * the baseclass for all things on screen. this classes are mainly there
 * to distribute events around from one widget to another and also to
 * handle psotions on screen (calculate local coordinated from global
 * and the other way around)
 */

#ifndef WIDGET__H_
#define WIDGET__H_

#include "rect.h"
#include "point.h"

#include "SDL/SDL.h"

class Widget {

  public:

    Widget(Widget* parentWidget, const Rect & r);
    virtual ~Widget();
    
    /**
     * if something happens this is where the widgets
     * hears the news
     * if the widget thinks nobody else needs to hear the
     * news return true else return false
     */
    virtual bool handleEvent(SDL_Event &e);
    
    /**
     * this function starts to send the given widget to the root object
     * in the widget tree and starts to distribute it around from there
     */
    void processEvent(SDL_Event &e);
    
    /**
     * returns true if the given absolute point is on this widget
     */
    bool onWidget(const Point &p) const;
    
  private:
  
    /**
     * inserts the widget into this widget
     */
    void addWidget(Widget *w);
    
    Rect position;
    Widget *parent;
    // point to the next widget that is situated in the same widgets on the same level
    Widget *next;
    // points to the last widget that was included into this widget
    Widget *last;
};

#endif
